import inspect

from matplotlib import pyplot as plt
import matplotlib as mpl

from .antidot import square, circle, diamond, triangle
from .lattice import honeycomb_lattice, square_lattice, hexagon_lattice

antidots = {
    "square": square,
    "circle": circle,
    "triangle": triangle,
    "diamond": diamond,
}
lattices = {
    "square": square_lattice,
    "honeycomb": honeycomb_lattice,
    "hexagonal": hexagon_lattice,
}


class adl:
    def __init__(
        self, a=100, ad=40, ring=10, reps=4, lattice="square", antidot="square"
    ):
        self.s = ""
        self.a = a
        self.ad_size = ad
        self.ring_width = ring
        self.reps = reps
        self.lattice_name = lattice
        self.antidot_name = antidot
        self.lattice = lattices[lattice](self.a, self.reps)
        self.antidot = antidots[antidot](self.ad_size, self.ring_width)
        self._Nx = reps * a
        self._Ny = reps * a
        self._Nz = 1
        self.pre()
        self.geom()
        self.post()

    def save(self, dir):
        name = f"{self.lattice_name}_lat_{self.antidot_name}_ad_{self.reps}x{self.reps}_ring_{self.ring_width}nm_ad_{self.ad_size}nm_a_{self.a}nm"
        self.s = inspect.cleandoc(self.s)
        with open(f"{dir}/{name}.mx3", "w") as f:
            f.writelines(self.s)

    def preview(self):
        fig, ax = plt.subplots(figsize=(3, 3))
        ax.set_xlim(0, self.a * self.reps)
        ax.set_ylim(0, self.a * self.reps)
        ax.patches.append(
            mpl.patches.Rectangle(
                (0, 0),
                self.a * self.reps,
                self.a * self.reps,
                color="k",
                transform=ax.transData,
            )
        )
        for x, y in self.lattice.coordinates:
            patch = self.antidot.get_patch(x, y, self.ad_size + self.ring_width)
            patch.set(transform=ax.transData, lw=0, facecolor="gray")
            ax.patches.append(patch)
        for x, y in self.lattice.coordinates:
            patch = self.antidot.get_patch(x, y, self.ad_size)
            patch.set(transform=ax.transData, lw=0, facecolor="w")
            ax.patches.append(patch)
        ax.set(xlabel="x (nm)", ylabel="y (nm)")
        fig.tight_layout()

    def pre(self):
        self.s += f"""
        dx := 1e-9
        dy := 1e-9
        dz := 4e-9
        Nx := {self._Nx}
        Ny := {self._Ny}
        Nz := {self._Nz}
        setgridsize(Nx,Ny, Nz)
        setcellsize(dx, dy, dz)
        edgesmooth = 8
        setPBC(32, 32, 0)

        // Py stripe
        adl := rect(Nx*dx,Ny*dy)
        m = uniform(0, 0, 1)
        Msat = 810e3
        aex = 13e-12
        alpha = 0.01
        Ku1 = 453195
        anisU = vector(0, 0, 1)
        alpha = 0.01

        """

    def geom(self):
        self.s += self.antidot.s

        self.s += """
        p := Nx*dx/2
            """
        for i, (x, y) in enumerate(self.lattice.coordinates):
            if i == 0:
                q = ":"
            else:
                q = ""
            self.s += f"""
        inner_dot {q}= inner_geom.transl(-p+{x:.4g},p-{y:.4g},0)
        outer_dot {q}= outer_geom.transl(-p+{x:.4g},p-{y:.4g},0)
        adl = adl.add(outer_dot)
        adl = adl.sub(inner_dot)
        m.setInShape(outer_dot, vortex(1, -1).transl(-p+{x:.4g},p-{y:.4g},0))
        defregion(1, outer_dot)
        Ku1.SetRegion(1, 0)

                    """

    def post(self):
        self.s += """
        setgeom(adl)
        setsolver(4)
        maxerr = 5e-5
        MinimizerStop = 5e-7
        minimize()
        relax()
        run(1e-10)
        relax()
        saveAs(m, "stable")

        Amps := 0.5e-4
        f_cut := 20e09
        t_sampl := 0.5 / (f_cut * 1.4)
        t0 := 50 / f_cut
        Maxdt = t_sampl / 100
        B_ext = vector(Amps*sinc(2*pi*f_cut*(t-t0)), 0, 0)

        tableadd(B_ext)
        tableautosave(t_sampl)
        autosave(m, t_sampl)

        run(1500 * t_sampl)
        """



# import math


class square_lattice:
    def __init__(self, a, reps):
        self.a = a
        self.reps = reps
        self.Nx = reps * self.a
        self.Ny = reps * self.a
        self.coordinates = []
        for i in range(self.reps):
            for j in range(self.reps):
                x = i * self.a + self.a / 2
                y = j * self.a + self.a / 2
                self.coordinates.append((x, y))


class hexagon_lattice:
    def __init__(self, a, reps):
        self.a = a
        self.reps = reps
        # self.h = a * math.sin(60 * math.pi / 180)
        self.Nx = reps * self.a
        self.Ny = reps * self.a
        self.coordinates = []
        for i in range(self.reps):
            for j in range(self.reps + 1):
                x = i * self.a + self.a / 2
                y = j * self.a + self.a / 2
                if i % 2 == 1:
                    y -= self.a / 2
                self.coordinates.append((x, y))


class honeycomb_lattice:
    def __init__(self, a, reps):
        self.a = a
        self.reps = reps
        self.Nx = reps * self.a
        self.Ny = reps * self.a
        self.coordinates = []
        for i in range(self.reps):
            for j in range(self.reps + 1):
                x = i * self.a + self.a / 2
                y = j * self.a + self.a / 2
                if i % 2 == 1:  # rows 1,3,5 ...
                    y -= self.a / 2
                    if j % 3 == 1:  # col 1,4,7 ...
                        continue
                if i % 2 == 0:  # rows 0,2,4 ...
                    if j % 3 == 2:  # col 2,5,8 ...
                        continue
                self.coordinates.append((x, y))



from setuptools import find_packages, setup

setup(
    name="adl",
    version="0.1.2",
    description="script generator for mumax3 for antidot lattice",
    author="Mathieu Moalic",
    author_email="matmoa@pm.me",
    platforms=["any"],
    license="GPL-3.0",
    url="https://github.com/MathieuMoalic/adl",
    packages=find_packages(),
    install_requires=[i.strip() for i in open("requirements.txt").readlines()],
)



from math import sqrt
import matplotlib as mpl


class square:
    def __init__(self, ad_size, ring_width):
        self.ad_size = ad_size
        self.ring_width = ring_width
        self.s = f"""// Square
        inner_geom := rect({ad_size:.4g},{ad_size:.4g})
        outer_geom := rect({ad_size + ring_width * 2:.4g},{ad_size + ring_width * 2:.4g})
        """

    def get_patch(self, center_x, center_y, ad_size):
        return mpl.patches.Rectangle(
            (center_x - ad_size / 2, center_y - ad_size / 2), ad_size, ad_size
        )


class diamond:
    def __init__(self, ad_size, ring_width):
        self.s = f"""// Diamond
        inner_geom := rect({ad_size:.4g},{ad_size:.4g}).RotZ(pi/4)
        outer_geom := rect({ad_size + ring_width * 2:.4g},{ad_size + ring_width * 2:.4g}).RotZ(pi/4)
        """

    def get_patch(self, center_x, center_y, ad_size):
        r = sqrt(2) * ad_size
        verts = [
            [center_x, center_y + r / 2],
            [center_x - r / 2, center_y],
            [center_x, center_y - r / 2],
            [center_x + r / 2, center_y],
            [center_x, center_y + r / 2],
        ]
        return mpl.patches.PathPatch(mpl.path.Path(verts, closed=True))


class circle:
    def __init__(self, ad_size, ring_width):
        self.s = f"""// Circle
        inner_geom := cylinder({ad_size:.4g},Nz*dz)
        outer_geom := cylinder({ad_size + ring_width * 2:.4g},Nz*dz)
        """

    def get_patch(self, center_x, center_y, ad_size):
        return mpl.patches.Circle((center_x, center_y), radius=ad_size / 2)


class triangle:
    def __init__(self, ad_size, ring_width):
        ad_size_outer = ad_size + ring_width * 2
        self.s = f"""// Triangle
        rec_right := rect({ad_size:.4g}, {ad_size * 2:.4g}).RotZ(pi/6).transl({3/4*ad_size:.4g},{(2-sqrt(3))*ad_size:.4g}/4,0)
        rec_left := rect({ad_size:.4g}, {ad_size * 2:.4g}).RotZ(-pi/6).transl({-3/4*ad_size:.4g},{(2-sqrt(3))*ad_size:.4g}/4,0)
        inner_geom := rect({ad_size:.4g}, {ad_size:.4g}).sub(rec_right).sub(rec_left)
        rec_right = rect({ad_size_outer:.4g}, {ad_size_outer * 2:.4g}).RotZ(pi/6).transl({3/4*ad_size_outer:.4g},{(2-sqrt(3))*ad_size_outer:.4g}/4,0)
        rec_left = rect({ad_size_outer:.4g}, {ad_size_outer * 2:.4g}).RotZ(-pi/6).transl({-3/4*ad_size_outer:.4g},{(2-sqrt(3))*ad_size_outer:.4g}/4,0)
        outer_geom := rect({ad_size_outer:.4g}, {ad_size_outer:.4g}).sub(rec_right).sub(rec_left).transl(0,4e-9,0)
        """

    def get_patch(self, center_x, center_y, ad_size):
        r = sqrt(3) * ad_size / 3
        verts = [
            [center_x, center_y + r],
            [center_x - ad_size / 2, center_y - r / 2],
            [center_x + ad_size / 2, center_y - r / 2],
            [center_x, center_y + r],
        ]
        return mpl.patches.PathPatch(mpl.path.Path(verts, closed=True))
